@@ -4,21 +4,23 @@
  * Version 2 by Charles Cazabon <charlesc-memtester@pyropus.ca>
  * Version 3 not publicly released.
  * Version 4 rewrite:
- * Copyright (C) 2004-2010 Charles Cazabon <charlesc-memtester@pyropus.ca>
+ * Copyright (C) 2004-2020 Charles Cazabon <charlesc-memtester@pyropus.ca>
  * Licensed under the terms of the GNU General Public License version 2 (only).
  * See the file COPYING for details.
  *
- * This file contains typedefs and structure definitions.
+ * This file contains typedefs, structure, and union definitions.
  *
  */
 
+#include "sizes.h"
+
 typedef unsigned long ul;
 typedef unsigned long long ull;
 typedef unsigned long volatile ulv;
+typedef unsigned char volatile u8v;
+typedef unsigned short volatile u16v;
 
-struct test
-{
+struct test {
     char *name;
     int (*fp)();
 };
-
